@@ -13,7 +13,7 @@ sys.path.append("@FENCEAGENTSLIBDIR@")
 from fencing import fail_usage, run_command, atexit_handler, check_input, process_input, show_docs, fence_action, all_opt
 from fencing import run_delay
 
-STORE_PATH = "/var/run/cluster/fence_scsi"
+STORE_PATH = "@STORE_PATH@"
 
 
 def get_status(conn, options):
@@ -84,14 +84,14 @@ def set_status(conn, options):
 # check if host is ready to execute actions
 def do_action_monitor(options):
 	# Check if required binaries are installed
-	if bool(run_cmd(options, options["--sg_persist-path"] + " -V")["err"]):
+	if bool(run_cmd(options, options["--sg_persist-path"] + " -V")["rc"]):
 		logging.error("Unable to run " + options["--sg_persist-path"])
 		return 1
-	elif bool(run_cmd(options, options["--sg_turs-path"] + " -V")["err"]):
+	elif bool(run_cmd(options, options["--sg_turs-path"] + " -V")["rc"]):
 		logging.error("Unable to run " + options["--sg_turs-path"])
 		return 1
 	elif ("--devices" not in options and 
-			bool(run_cmd(options,	options["--vgs-path"] + " --version")["err"])):
+			bool(run_cmd(options, options["--vgs-path"] + " --version")["rc"])):
 		logging.error("Unable to run " + options["--vgs-path"])
 		return 1
 
@@ -102,11 +102,13 @@ def do_action_monitor(options):
 	return 0
 
 
-#run command, returns dict, ret["err"] = exit code; ret["out"] = output
+# run command, returns dict, ret["rc"] = exit code; ret["out"] = output;
+# ret["err"] = error
 def run_cmd(options, cmd):
 	ret = {}
-	(ret["err"], ret["out"], _) = run_command(options, cmd)
+	(ret["rc"], ret["out"], ret["err"]) = run_command(options, cmd)
 	ret["out"] = "".join([i for i in ret["out"] if i is not None])
+	ret["err"] = "".join([i for i in ret["err"] if i is not None])
 	return ret
 
 
@@ -122,11 +124,11 @@ def is_block_device(dev):
 def preempt_abort(options, host, dev):
 	reset_dev(options,dev)
 	cmd = options["--sg_persist-path"] + " -n -o -A -T 5 -K " + host + " -S " + options["--key"] + " -d " + dev
-	return not bool(run_cmd(options, cmd)["err"])
+	return not bool(run_cmd(options, cmd)["rc"])
 
 
 def reset_dev(options, dev):
-	return run_cmd(options, options["--sg_turs-path"] + " " + dev)["err"]
+	return run_cmd(options, options["--sg_turs-path"] + " " + dev)["rc"]
 
 
 def register_dev(options, dev):
@@ -135,28 +137,31 @@ def register_dev(options, dev):
 		for slave in get_mpath_slaves(dev):
 			register_dev(options, slave)
 		return True
+	if get_reservation_key(options, dev, False) == options["--key"]:
+		return True
 	reset_dev(options, dev)
 	cmd = options["--sg_persist-path"] + " -n -o -I -S " + options["--key"] + " -d " + dev
 	cmd += " -Z" if "--aptpl" in options else ""
 	#cmd return code != 0 but registration can be successful
-	return not bool(run_cmd(options, cmd)["err"])
+	return not bool(run_cmd(options, cmd)["rc"])
 
 
 def reserve_dev(options, dev):
 	reset_dev(options,dev)
 	cmd = options["--sg_persist-path"] + " -n -o -R -T 5 -K " + options["--key"] + " -d " + dev
-	return not bool(run_cmd(options, cmd)["err"])
+	return not bool(run_cmd(options, cmd)["rc"])
 
 
-def get_reservation_key(options, dev):
+def get_reservation_key(options, dev, fail=True):
 	reset_dev(options,dev)
 	opts = ""
 	if "--readonly" in options:
 		opts = "-y "
 	cmd = options["--sg_persist-path"] + " -n -i " + opts + "-r -d " + dev
 	out = run_cmd(options, cmd)
-	if out["err"]:
-		fail_usage("Cannot get reservation key")
+	if out["rc"] and fail:
+		fail_usage('Cannot get reservation key on device "' + dev
+                        + '": ' + out["err"])
 	match = re.search(r"\s+key=0x(\S+)\s+", out["out"], re.IGNORECASE)
 	return match.group(1) if match else None
 
@@ -169,8 +174,9 @@ def get_registration_keys(options, dev, fail=True):
 		opts = "-y "
 	cmd = options["--sg_persist-path"] + " -n -i " + opts + "-k -d " + dev
 	out = run_cmd(options, cmd)
-	if out["err"]:
-		fail_usage("Cannot get registration keys", fail)
+	if out["rc"]:
+		fail_usage('Cannot get registration keys on device "' + dev
+                        + '": ' + out["err"], fail)
 		if not fail:
 			return []
 	for line in out["out"].split("\n"):
@@ -257,6 +263,7 @@ def dev_write(dev, options):
 		f = open(file_path, "a+")
 	except IOError:
 		fail_usage("Failed: Cannot open file \""+ file_path + "\"")
+	f.seek(0)
 	out = f.read()
 	if not re.search(r"^" + dev + "\s+", out, flags=re.MULTILINE):
 		f.write(dev + "\n")
@@ -277,11 +284,6 @@ def dev_read(fail=True):
 	return devs
 
 
-def dev_delete(options):
-	file_path = options["store_path"] + ".dev"
-	os.remove(file_path) if os.path.exists(file_path) else None
-
-
 def get_clvm_devices(options):
 	devs = []
 	cmd = options["--vgs-path"] + " " +\
@@ -291,7 +293,7 @@ def get_clvm_devices(options):
 	"--options vg_attr,pv_name "+\
 	"--config 'global { locking_type = 0 } devices { preferred_names = [ \"^/dev/dm\" ] }'"
 	out = run_cmd(options, cmd)
-	if out["err"]:
+	if out["rc"]:
 		fail_usage("Failed: Cannot get clvm devices")
 	for line in out["out"].split("\n"):
 		if 'c' in line.split(":")[0]:
